{************************************************************************
*  this is a demo program for dOPCEventServer Component
*
*  Copyright  2002-2019  Dipl.-Ing. H.-D. Kassl  All rights reserved.
*
*  http://www.dopc.kassl.de
*  e-mail: dopc@kassl.de
*
*************************************************************************}

unit dOPCAEDemoMain;

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ImgList, ExtCtrls, Menus, ComCtrls,
  dOPCCom, dOPCComn, dOPCAE, dOPCEventSortDlg,dOPCDefFilterDlg,
  dOPCDlgServerSelect, dOPCConditionStateDlg, Buttons, dOPCIntf,
  System.ImageList;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabServer: TTabSheet;
    TabAlarms: TTabSheet;
    Filter: TGroupBox;
    canFilterByEvent: TCheckBox;
    canFilterByCategory: TCheckBox;
    canFilterBySEVERITY: TCheckBox;
    canFilterByArea: TCheckBox;
    canFilterBySource: TCheckBox;
    ListView1: TListView;
    ImageList1: TImageList;
    Panel1: TPanel;
    Panel2: TPanel;
    Label1: TLabel;
    eEventType: TEdit;
    Label2: TLabel;
    eEventCategory: TEdit;
    eCondition: TEdit;
    Label3: TLabel;
    eActiveTime: TEdit;
    Label4: TLabel;
    cAcknowledge: TCheckBox;
    Label5: TLabel;
    eEventId: TEdit;
    Label6: TLabel;
    eActorId: TEdit;
    Label7: TLabel;
    eQuality: TEdit;
    eSubCondition: TEdit;
    Label8: TLabel;
    cEnabled: TCheckBox;
    cActive: TCheckBox;
    cACKED: TCheckBox;
    PopupMenu1: TPopupMenu;
    Acknowledge1: TMenuItem;
    AcknowledgewithComment1: TMenuItem;
    AcknowledgeAll1: TMenuItem;
    dOPCEventServer1: TdOPCEventServer;
    Memo2: TMemo;
    MainMenu1: TMainMenu;
    File1: TMenuItem;
    Connect1: TMenuItem;
    Disconnect1: TMenuItem;
    N1: TMenuItem;
    Exit1: TMenuItem;
    Label9: TLabel;
    Panel3: TPanel;
    Memo1: TMemo;
    bClear: TButton;
    GroupBox1: TGroupBox;
    Label10: TLabel;
    lVendor: TLabel;
    State: TLabel;
    lState: TLabel;
    Label11: TLabel;
    lVersion: TLabel;
    Label13: TLabel;
    lStarttime: TLabel;
    Edit1: TMenuItem;
    Freeze1: TMenuItem;
    Refresh1: TMenuItem;
    N2: TMenuItem;
    AcknownledgeEvent1: TMenuItem;
    Acknownledgeallnotactive1: TMenuItem;
    Help1: TMenuItem;
    Info1: TMenuItem;
    SortListBy1: TMenuItem;
    SetFilter1: TMenuItem;
    N3: TMenuItem;
    bCondState: TSpeedButton;
    procedure FormDestroy(Sender: TObject);
    procedure ListView1Change(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure ListView1CustomDrawItem(Sender: TCustomListView;
      Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
    procedure FormShow(Sender: TObject);
    procedure Acknowledge1Click(Sender: TObject);
    procedure AcknowledgeAll1Click(Sender: TObject);
    procedure AcknowledgewithComment1Click(Sender: TObject);
    procedure dOPCEventServer1Connect(Sender: TObject);
    procedure dOPCEventServer1Disconnect(Sender: TObject);
    procedure dOPCEventServer1ServerShutdown(Sender: TObject; Reason: string);
    procedure Freeze1Click(Sender: TObject);
    procedure Connect1Click(Sender: TObject);
    procedure Exit1Click(Sender: TObject);
    procedure Disconnect1Click(Sender: TObject);
    procedure bClearClick(Sender: TObject);
    procedure Refresh1Click(Sender: TObject);
    procedure Info1Click(Sender: TObject);
    procedure dOPCEventServer1EndUpdate(Sender: TObject);
    procedure SortListBy1Click(Sender: TObject);
    procedure SetFilter1Click(Sender: TObject);
    procedure bCondStateClick(Sender: TObject);
    procedure ListView1DblClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormResize(Sender: TObject);
  private
    SelectedEventID : integer;
    procedure AckEvent(AckName, AckComment: string);
    procedure UpdateListView;
    function  GetEvent(ListItem: TListItem): tdOPCEvent;
    procedure ClearInfos(aTag : integer);
    procedure AddNoConnectInfo;
  public
  end;

var
  MainForm: TMainForm;

implementation
{$R *.DFM}

procedure TMainForm.FormShow(Sender: TObject);
begin
  SelectedEventId := -1;   // no event is selected
  ClearInfos(1);           // clear all info fields
  ClearInfos(2);           // clear all info fields
  AddNoConnectInfo;
  PageControl1.ActivePage := TabAlarms;
end;

//-----------------------------------------------------------------------------
procedure TMainForm.FormDestroy(Sender: TObject);
begin
   dOPCEventServer1.Active := false;  // if active -> disconnect
end;

//-----------------------------------------------------------------------------
// get event data from list item data
// DO NOT USE POINTER
function TMainForm.GetEvent(ListItem: TListItem) : tdOPCEvent;
begin
  result := dOPCEventServer1.Events.ById(Integer(ListItem.Data));
end;

procedure TMainForm.AddNoConnectInfo;
var
 Item  : TListItem;
begin
  Item := ListView1.Items.Insert(0);               // add new view item
  Item.Data    := nil;
  Item.SubItems.Clear;
  Item.Caption := FormatDateTime('dd.mm hh:nn:ss',Now);
  Item.SubItems.Add('dOPC demo client');
  Item.SubItems.Add('not connected to server');
  Item.SubItems.Add('1000');
end;

//-----------------------------------------------------------------------------
// this method put all events to the listview
procedure TMainForm.UpdateListView;
var
 Item  : TListItem;
 Event : tdOPCEvent;
 i     : integer;
 FocusIsSet: boolean;
begin
 if Freeze1.Checked then exit;     // if freeze is checked -> do nothing
 FocusIsSet := false;              // we have no selected item

 ListView1.Items.BeginUpdate;      // begin with fill out listview
 ListView1.AllocBy := dOPCEventServer1.Events.Count+1;
 try
   ListView1.Items.Clear;               // first clear all items
   if dOPCEventServer1.Active = false then
     AddNoConnectInfo;
   for i := 0 to dOPCEventServer1.Events.Count-1 do   // for all event in list
   begin
      Event :=  dOPCEventServer1.Events[i];           // get event from list
      Item := ListView1.Items.Add;                    // add new view item
      Item.Data    := pointer(Event.Id);              // store event ID to item
      Item.SubItems.Clear;
      Item.Caption := FormatDateTime('dd.mm hh:nn:ss',Event.Time);
      Item.SubItems.Add(Event.Source);
      if Event.Active then                        // if event not active
      begin
        Item.ImageIndex := 0;                          // show yellow bell
        Item.SubItems.Add(Event.Msg);                  // show current message
      end
      else
      begin
        Item.SubItems.Add(Event.LastEvent.Msg);        // show the old message
        Item.ImageIndex := 1;                          // show red bell
      end;
      if Event.Acknowledged then
      begin
        if Item.ImageIndex = 0 then
          Item.ImageIndex := 2
        else
          Item.ImageIndex := 3
      end;
      Item.SubItems.Add(IntToStr(Event.Severity));
      if Event.ID = SelectedEventID then              // if this event is selected event
      begin
        Item.Selected := true;                        // select this item
        Item.Focused  := true;
        FocusIsSet    := true;                        // we have selected one item
      end;
   end;

   if not FocusIsSet then                             // if no item is selected
   begin
     if ListView1.Items.Count > 0 then                // if items in list
     begin                                            // select the first item
       ListView1.Items[0].Selected := true;
       ListView1.Items[0].Focused  := true;
       if ListView1.CanFocus then                     // if visible
         ListView1.SetFocus;                          // also set focus
     end;
   end;
 finally
   ListView1.Items.EndUpdate;
 end;
end;

//-----------------------------------------------------------------------------
// show more information for selected event
procedure TMainForm.ListView1Change(Sender: TObject; Item: TListItem;
  Change: TItemChange);
var
 ListItem: TListItem;
 Event   : TdOPCEvent;
 i : integer;
begin
  ListItem := ListView1.Selected;        // get selected item
  if ListItem <> nil then                // if one item is selected
  begin                                  // show information
    Event := GetEvent(ListItem);         // first get event from item data
    if Event = nil then exit;            // if not event -> exit
    SelectedEventID      := Event.ID;    // save selected event
    eEventType.Text      := Event.EventTypeStr;
  //  eEventCategory.Text  := IntToStr(Event.EventCategory);
    eEventCategory.Text  := Event.CategoryName;
    eCondition.Text      := Event.ConditionName;
    eSubCondition.Text   := Event.SubconditionName;
    eActiveTime.Text     := DateTimeToStr(Event.ActiveTime);
    cAcknowledge.checked := Event.AckRequired;
    eEventId.Text        := IntToStr(Event.Cookie);
    eActorId.Text        := Event.ActorId;
    if Event.IsGoodQuality then
      eQuality.Text := 'Good'
    else
      eQuality.Text := 'Bad';
    cEnabled.checked := Event.Enabled;
    cActive.checked  :=  Event.Active;
    cAckEd.checked   :=  Event.Acknowledged;
    Memo2.Lines.BeginUpdate;
    Memo2.Lines.clear;
    for i := 0 to Event.CustomProps.Count- 1 do  // for all custom properties
       // show custom property name and value
       Memo2.Lines.Add(Event.CustomProps.Names[i]+ '->'+ Event.CustomProps.ValueAsString[i]);
    Memo2.Lines.EndUpdate
  end
  else
    ClearInfos(2);   // nothing is selected -> so, show no info
end;

//-----------------------------------------------------------------------------
// it looks better with color
procedure TMainForm.ListView1CustomDrawItem(Sender: TCustomListView;
  Item: TListItem; State: TCustomDrawState; var DefaultDraw: Boolean);
var
  Event : TdOPCEvent;
begin
  Event := GetEvent(Item);
  if Event <> nil then
  begin
    if Event.Active then                      // if active event
      ListView1.Canvas.Font.Color := clred    // red color
    else
      ListView1.Canvas.Font.Color := clgreen; // green color
   // if Event.Acknowledged then
   //      ListView1.Canvas.Font.Color := clblack;
  end
  else
    ListView1.Canvas.Font.Color := clred;    // red color
  DefaultDraw := true;                       // make default procedure
end;

//-----------------------------------------------------------------------------
// user want to acknowledge selected Event
procedure TMainForm.AckEvent(AckName, AckComment: string);
var
 ListItem: TListItem;
 Event   : TdOPCEvent;
begin
  ListItem := ListView1.Selected;            // get selected list item
  if ListItem <> nil then                    // if one item selected
  begin
    Event := GetEvent(ListItem);             // get event from item data
    if Event <> nil then
    begin
      Event.Acknowledge(AckName,AckComment);  // now acknowledge event
    end
  end;
end;

//-----------------------------------------------------------------------------
procedure TMainForm.Acknowledge1Click(Sender: TObject);
begin
  AckEvent('Kassl','');
end;

//-----------------------------------------------------------------------------
procedure TMainForm.AcknowledgeAll1Click(Sender: TObject);
begin
  if dOPCEventServer1.Active then
    dOPCEventServer1.AcknowledgeNotActiveEvents('ALL','Acknowledge all');
end;

//-----------------------------------------------------------------------------
procedure TMainForm.AcknowledgewithComment1Click(Sender: TObject);
var
 AckName : string;
begin   
  if InputQuery('Acknowledge Event','Acknowledge by',AckName) then
    AckEvent(AckName,'no comment');
end;

//-----------------------------------------------------------------------------
// OnConnect event from OPCEventserver
procedure TMainForm.dOPCEventServer1Connect(Sender: TObject);
var
  ServerInfo: tServerStateRec;
begin
  Memo1.Lines.Add('connect to OPC AE server: '+dOPCEventServer1.ServerCaption);

  // just for info (show filter options)
  CanFilterByEvent.Checked    := dOPCEventServer1.CanFilterByEvent;
  CanFilterByCategory.Checked := dOPCEventServer1.CanFilterByCategory;
  CanFilterBySeverity.Checked := dOPCEventServer1.CanFilterBySeverity;
  CanFilterByArea.Checked     := dOPCEventServer1.CanFilterByArea;
  CanFilterBySource.Checked   := dOPCEventServer1.CanFilterBySource;

  // fill out server information screen
  dOPCEventServer1.GetServerInfo(ServerInfo);
  with ServerInfo do
  begin
    Lstate.Caption     := dOPCServerStateToStr(ServerState,coCOM);
    lVendor.Caption    := VendorInfo;
    lVersion.Caption   := format('%d.%d (%s)',[MajorVersion,MinorVersion,BuildNumber]);
    lStarttime.Caption := DateTimetoStr(Starttime);
  end;
end;

//-----------------------------------------------------------------------------
// OnDisconnect event from OPCEventserver
procedure TMainForm.dOPCEventServer1Disconnect(Sender: TObject);
begin
  ClearInfos(1);
  ClearInfos(2);
  AddNoConnectInfo;
  Memo1.Lines.Add('disconnect from OPC AE server: '+dOPCEventServer1.ServerCaption);
end;

//-----------------------------------------------------------------------------
// OnServerShutdownEvent from OPCEventserver
procedure TMainForm.dOPCEventServer1ServerShutdown(Sender: TObject; Reason: String);
begin
  Memo1.Lines.Add('server shutdown connection '+ reason);
end;

//-----------------------------------------------------------------------------
procedure TMainForm.Freeze1Click(Sender: TObject);
begin
  Freeze1.Checked := not Freeze1.Checked;
  if not Freeze1.Checked then
      UpdateListView;              // show all event in listview
end;

//-----------------------------------------------------------------------------
// clear all info labels and editfields
procedure TMainForm.ClearInfos(aTag : integer);
var
  i : integer;
  C : TComponent;
begin
  for i := 0 to ComponentCount-1 do        // clears all Infolabels
  begin
    C := Components[i];
    if C.Tag = aTag then
    begin
      if C is TLabel    then TLabel(C).Caption := '';
      if C is TEdit     then TEdit(C).Text := '';
      if C is TCheckBox then TCheckBox(C).Checked := false;
    end;
  end;
  if aTag = 1 then
    lState.Caption := 'not connected';
  if aTag = 2 then
    Memo2.clear;
end;

//-----------------------------------------------------------------------------
// we want to connect to a new opc server
procedure TMainForm.Connect1Click(Sender: TObject);
begin
  if dOPCSelectAEServerDlg(dOPCEventServer1)  then // call dialog
  begin   // if the user has selected a new opc ae server
    SelectedEventId := -1;                          // no event is selected
    Listview1.Items.clear;                          // delete all items from list
    Freeze1.Checked := false;                       // no freeze at beginning
    dOPCEventServer1.Active := true;                // connect to server
  end;
end;

//-----------------------------------------------------------------------------
// OnEndUpdate from dOPCEventServer
// we have new events from server
procedure TMainForm.dOPCEventServer1EndUpdate(Sender: TObject);
begin
  if not Freeze1.Checked then         // if not freeze
    UpdateListView;                   // show all event in listview
end;

procedure TMainForm.SortListBy1Click(Sender: TObject);
begin
  dOPCDefineSortOrder(dOPCEventserver1);
end;

procedure TMainForm.SetFilter1Click(Sender: TObject);
begin
  dOPCDefineFilter(dOPCEventserver1);
end;


//-----------------------------------------------------------------------------
// the user want to exit the program
procedure TMainForm.Exit1Click(Sender: TObject);
begin
  close;
end;

//-----------------------------------------------------------------------------
// the user want to disconnect from opc server
procedure TMainForm.Disconnect1Click(Sender: TObject);
begin
  dOPCEventServer1.Active := false;
end;

procedure TMainForm.Refresh1Click(Sender: TObject);
begin
 dOPCEventServer1.Refresh;
end;

//-----------------------------------------------------------------------------
// clear info list
procedure TMainForm.bClearClick(Sender: TObject);
begin
  Memo1.Clear;
end;


procedure TMainForm.Info1Click(Sender: TObject);
begin
   MessageBox(Handle,'dOPC A&E Client Version 1.1'+LF+
                    'This program is freeware.'+LF+
                    'Developed with dOPC Client Toolkit for Delphi version '+dOPCVersion+LF
                     +LF+
                    '(c) 2002-2020 Kassl GmbH (http://www.kassl.de)'+LF+
                    ''+LF+
                    'THIS SOFTWARE IS DISTRIBUTED "AS IS" AND WITHOUT'+LF+
                    'WARRANTIES AS TO PERFORMANCE OF MERCHANTABILITY '+LF+
                    'OR ANY OTHER WARRANTIES WHETHER EXPRESSED'+LF+
                    'OR IMPLIED. YOU USE IT AT YOUR OWN RISK. '+LF+
                    'THE AUTHOR WILL NOT BE LIABLE FOR DATA LOSS,'+LF+
                    'DAMAGES,  LOSS OF PROFITS OR ANY OTHER KIND OF'+LF+
                    'LOSS WHILE USING OR MISUSING THIS SOFTWARE.'+LF+
                    LF+
                    'The source code of this program is included in our'+LF+
                    'dOPC Client Toolkit for Delphi version '+dOPCVersion+LF
                    ,'About',
                    MB_OK or MB_ICONINFORMATION);
end;

procedure TMainForm.bCondStateClick(Sender: TObject);
var
 ListItem: TListItem;
 Event   : TdOPCEvent;
 Conditionstate: tdOPCEventCondition;
begin
  ListItem := ListView1.Selected;            // get selected list item
  if ListItem <> nil then                    // if one item selected
  begin
    Event := GetEvent(ListItem);             // get event from item data
    if Event = nil then exit;
    Conditionstate := Event.GetConditionState;
    if Conditionstate <> nil then
    begin
      ShowConditionState(Conditionstate);
      Conditionstate.Free;
    end;
  end;
end;


procedure TMainForm.ListView1DblClick(Sender: TObject);
begin
  bCondStateClick(Self);
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  dOPCInitSecurity;
end;

procedure TMainForm.FormResize(Sender: TObject);
begin
   Memo2.Width := Width - 15 - Memo2.Left; 
end;

end.
