unit dOPCDefFilterDlg;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ComCtrls, StdCtrls, ExtCtrls, Menus, dOPCAE;

type
  TdOPCDefineFilterDlg = class(TForm)
    FilterBox: TListBox;
    Panel2: TPanel;
    Panel1: TPanel;
    gFilter: TGroupBox;
    Label4: TLabel;
    Label1: TLabel;
    cActive: TCheckBox;
    eFilterName: TEdit;
    gEvents: TGroupBox;
    Label6: TLabel;
    Label7: TLabel;
    Label8: TLabel;
    cSimple: TCheckBox;
    cCondition: TCheckBox;
    cTracking: TCheckBox;
    gSeverity: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    eFrom: TEdit;
    eTo: TEdit;
    TabControl1: TTabControl;
    Listbox: TListBox;
    bFilterDef: TSpeedButton;
    Panel3: TPanel;
    bFilterAdd: TSpeedButton;
    bFilterDel: TSpeedButton;
    bUpdate: TButton;
    bClose: TButton;
    procedure FormShow(Sender: TObject);
    procedure bFilterDelClick(Sender: TObject);
    procedure bFilterAddClick(Sender: TObject);
    procedure FilterBoxClick(Sender: TObject);
    procedure bFilterDefClick(Sender: TObject);
    procedure ListboxDblClick(Sender: TObject);
    procedure TabControl1Change(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bUpdateClick(Sender: TObject);
    procedure cActiveClick(Sender: TObject);
    procedure eFilterNameChange(Sender: TObject);
  private
    Server: TdOPCEventServer;
    CatList  : TStringlist;
    AreaList : TStringlist;
    SourceList : TStringlist;
    procedure EnableEdits(doEnable: boolean);
    procedure ShowProperties(Filter: tdOPCEventFilter);
  public
  end;

function dOPCDefineFilter(Server: TdOPCEventServer): boolean;


implementation
uses
  dOPCSelAreaDlg,dOPCSelCategoryDlg,dOPCSelSourceDlg;

{$R *.DFM}

function dOPCDefineFilter(Server: TdOPCEventServer): boolean;
var
  D : TdOPCDefineFilterDlg;
begin
 result := false;
 Application.CreateForm(TdOPCDefineFilterDlg,D);
 D.Server  := Server;
 try
   if (D.ShowModal() = idOk) then
     result := true;
 finally
   D.free();
 end;
end;

procedure TdOPCDefineFilterDlg.FormCreate(Sender: TObject);
begin
  CatList    := TStringlist.create;
  AreaList   := TStringlist.create;
  SourceList := TStringlist.create;
end;

procedure TdOPCDefineFilterDlg.FormDestroy(Sender: TObject);
begin
  CatList.Free;
  AreaList.Free;
  SourceList.Free;
end;

procedure TdOPCDefineFilterDlg.FormShow(Sender: TObject);
var
 i : integer;
 Name : string;
begin
   for i := 0 to Server.Filter.Count-1 do
   begin
     Name := Server.Filter[i].Name;
     if trim(Name) = '' then
       Name := 'Filter_'+IntToStr(i+1);
     FilterBox.Items.Add(Name);
   end;
   if FilterBox.Items.Count > 0 then
     FilterBox.ItemIndex := 0;
   FilterBoxClick(self)

end;


procedure TdOPCDefineFilterDlg.EnableEdits(doEnable: boolean);
begin
   cActive.Enabled    := doEnable;
   eFilterName.Enabled    := doEnable;
   cSimple.Enabled    := doEnable;
   cCondition.Enabled    := doEnable;
   cTracking.Enabled    := doEnable;
   eFrom.Enabled    := doEnable;
   eTo.Enabled    := doEnable;

   gFilter.Enabled    := doEnable;
   gSeverity.Enabled   := doEnable;
   gEvents.Enabled     := doEnable;
   TabControl1.Enabled := doEnable;
   bFilterDef.Enabled  := doEnable;
   if not doEnable then
   begin
     cActive.Checked    := false;
     eFilterName.Text   := '';
     cSimple.Checked    := false;
     cCondition.Checked := false;
     cTracking.Checked  := false;
     eFrom.Text         := '';
     eTo.Text           := '';
   end;
end;


// show filter properties on form
procedure TdOPCDefineFilterDlg.ShowProperties(Filter :tdOPCEventFilter);
begin
  if not bFilterDef.Enabled then        // if form not enabled
    EnableEdits(true);                  // enabled form for user input
  cActive.Checked    := Filter.Active;
  eFilterName.Text   := Filter.Name;
  if Filter.EventType = [] then
  begin
    cSimple.Checked    := true;
    cCondition.Checked := true;
    cTracking.Checked  := true;
  end
  else
  begin
    cSimple.Checked    := evSimple    in Filter.EventType;
    cCondition.Checked := evCondition in Filter.EventType;
    cTracking.Checked  := evTracking  in Filter.EventType
  end;
  eFrom.Text         := IntToStr(Filter.SeverityLow);
  eTo.Text           := IntToStr(Filter.SeverityHigh);
  case TabControl1.TabIndex of
    0: CatList.Assign(Filter.Categories);
    1: AreaList.Assign(Filter.Areas);
    2: SourceList.Assign(Filter.Sources);
  end;
  TabControl1Change(self);
end;


// user want a new filter
procedure TdOPCDefineFilterDlg.bFilterAddClick(Sender: TObject);
var
  Filter : tdOPCEventFilter;
begin
   Filter := Server.Filter.Add;       // make new filter
   Filter.Active := false;
   FilterBox.Items.Add(Filter.Name);  // add filter name to listbox
   FilterBox.ItemIndex := FilterBox.Items.Count-1;
   ShowProperties(Filter);            // show properties on form
   bUpdate.Enabled := false;
end;

// user want to delete current selected filter
procedure TdOPCDefineFilterDlg.bFilterDelClick(Sender: TObject);
var
  SelIndex : integer;
begin
   SelIndex := FilterBox.ItemIndex;
   if SelIndex >= 0 then                  // if a filter is selected
   begin
     Server.Filter[SelIndex].Free;        // delete filter form dOPCEventserver Object
     FilterBox.Items.Delete(SelIndex);    // delete filter from listbox
     //Server.Filter.Refresh;               // Refresh all filter
     Server.Refresh;
   end;
   if SelIndex > 0 then
     FilterBox.ItemIndex := SelIndex-1;
   FilterBoxClick(Self);                  // show new filter properties
end;


// user has select a new filter
procedure TdOPCDefineFilterDlg.FilterBoxClick(Sender: TObject);
var
  SelIndex : integer;
begin
  SelIndex := FilterBox.ItemIndex;          // get selected filter
  if SelIndex >= 0 then                     // if filter is selected
    ShowProperties(Server.Filter[SelIndex]) // show properties of filter
  else                                      // if no filter is selected
    EnableEdits(false);                     // Disable properties
  bUpdate.Enabled := false;                 // no changes
end;

// user want set new Category or Area or Source filter
procedure TdOPCDefineFilterDlg.bFilterDefClick(Sender: TObject);
var
 Ok : boolean;
begin
  Ok := false;
  case TabControl1.TabIndex of
    0: OK := GetdOPCFilterCategories(Server,CatList);
    1: OK := GetdOPCFilterAreas(Server.Browser,AreaList);
    2: OK := GetdOPCFilterSources(Server.Browser,Sourcelist);
  end;
  if Ok then                   // if user has changed something
  begin
    bUpdate.Enabled := true;   // to store user has to press update button
    TabControl1Change(self);   // show filter items in form
  end;
end;

procedure TdOPCDefineFilterDlg.ListboxDblClick(Sender: TObject);
begin
  bFilterDefClick(self);
end;

procedure TdOPCDefineFilterDlg.TabControl1Change(Sender: TObject);
var
  SelIndex : integer;
//  Filter   : tdOPCEventFilter;
begin
  SelIndex := FilterBox.ItemIndex;
  if SelIndex >= 0 then
  begin
    //Filter := Server.Filter[SelIndex];
    case TabControl1.TabIndex of
      0: ListBox.Items.Assign(CatList);
      1: ListBox.Items.Assign(AreaList);
      2: ListBox.Items.Assign(SourceList);
    end;
  end;
end;

// user has changed something
procedure TdOPCDefineFilterDlg.cActiveClick(Sender: TObject);
begin
  bUpdate.Enabled := true;
end;

// user has changed something
procedure TdOPCDefineFilterDlg.eFilterNameChange(Sender: TObject);
begin
  bUpdate.Enabled := true;
end;


// update filter properties
procedure TdOPCDefineFilterDlg.bUpdateClick(Sender: TObject);
var
  SelIndex : integer;
  Filter   : tdOPCEventFilter;
begin
  SelIndex := FilterBox.ItemIndex;
  if SelIndex >= 0 then
  begin
    Filter      := Server.Filter[SelIndex];
    Filter.Name := eFilterName.Text;
    FilterBox.Items[SelIndex] := Filter.Name;
    Filter.Active := cActive.Checked;
    Filter.EventType := [];
    if cSimple.Checked then
      Filter.EventType :=  Filter.EventType + [evSimple];
    if cCondition.Checked then
      Filter.EventType :=  Filter.EventType + [evCondition];
    if cTracking.Checked then
      Filter.EventType :=  Filter.EventType + [evTracking];

    if cSimple.Checked and
       cCondition.Checked and
       cTracking.Checked then
      Filter.EventType := [];
    Filter.SeverityLow  := StrToInt(eFrom.Text);
    Filter.SeverityHigh := StrToInt(eTo.Text);
    Filter.Categories.Assign(CatList);
    Filter.Areas.Assign(AreaList);
    Filter.Sources.Assign(SourceList);
    Filter.Refresh;
    bUpdate.Enabled := false;
    Server.Refresh;
  end;
end;


end.
